define([
	"toolbox"
]
, function (Toolbox) {
	var that
	, typeMap = {
		1:	{
			length:		1
			, unpack:	Types.getInt8
		}

		, 2:	{
			length:		1
			, unpack:	Types.getUint8
		}
		
		, 3:	{
			length:		2
			, unpack:	Types.getInt16
		}
		
		, 4:	{
			length:		2
			, unpack:	Types.getUint16
		}
		
		, 5:	{
			length:		4
			, unpack:	Types.getInt32
		}
		
		, 6:	{
			length:		4
			, unpack:	Types.getUint32
		}
		
		, 7:	{
			length:		8
			, unpack:	Types.getFloat64
		}
		
		// There is no unsigned 64bit integer in JS.
		, 8:	{
			length:		8
			, unpack:	Types.getFloat64
		}
		
		, 9:	{
			length:		null
			, unpack:	Types.getUTFString
		}
		
		, 10:	{
			length:		4
			, unpack:	Types.getDate
		}
		
		, 11:	{
			length:		2
			, unpack:	Types.getVersion
		}
		
		, 12:	{
			length:		null
			, unpack:	Types.getCollection
		}
	}

	, Types		= Toolbox.Base.extend({
		constructor:	function (code) {
			this.code		= code
			, this.length	= null
			;

			if (_.isNumber(code)) {
				if (!_.isUndefined(typeMap[code])) {
					_.extend(this, typeMap[code]);
				} else {

				}
			}

			return this;
		}

		, code:		null
		, length:	null
		, unpack:	null

		, getCode:			function() {
			return this.code;
		}
	
		, getByteLength:	function() {
			return this.length;
		}
	}
	, {
		getInt8:				function (chunk) {
			return new DataView(chunk.slice(0,typeMap[1].length)).getInt8(0, false);
		}

		, getUint8:				function (chunk) {
			return new DataView(chunk.slice(0,typeMap[2].length)).getUint8(0, false);
		}

		, getInt16:				function (chunk) {
			return new DataView(chunk.slice(0,typeMap[3].length)).getInt16(0, false);
		}

		, getUint16:			function (chunk) {
			return new DataView(chunk.slice(0,typeMap[4].length)).getUint16(0, false);
		}

		, getInt32:				function (chunk) {
			return new DataView(chunk.slice(0,typeMap[5].length)).getInt32(0, false);
		}

		, getUint32:			function (chunk) {
			return new DataView(chunk.slice(0,typeMap[6].length)).getUint32(0, false);
		}

		, getFloat64:			function (chunk) {
			return new DataView(chunk.slice(0,typeMap[7].length)).getFloat64(0, false);
		}

		, getString:			function(chunk, length) {
			if (!_.isNumber(length) || length > chunk.byteLength) {
				var length	= chunk.byteLength;
			}
			
			return String.fromCharCode.apply(null, new Uint8Array(chunk.slice(0, length)));
		}

		// TODO: This might be optimizable
		, getUTFString:			function (chunk, length) {
			var str = '';
			if (!_.isNumber(length) || length > chunk.byteLength) {
				var length	= chunk.byteLength;
			}
			
			for (var i = 0; i < length; i++) {
				var slice	= chunk.slice(i, i+1);
				var num		= new DataView(slice).getUint8(0, false);
				
				str += num <= 0x7F ?
					num === 0x25 ? "%25" :
					String.fromCharCode(num) :
					"%" + num.toString(16);
			}
			
			return decodeURIComponent(str);
		}

		, getDate:				function (chunk) {
			return new Date(Types.getUint32(chunk));
		}

		, getVersion:			function (chunk) {
			return Types.getInt8(chunk.slice(0,1)) + "." + Types.getInt8(chunk.slice(1,2));
		}
		
		, getModel:				function (chunk) {
			return chunk;
		}

		, getUnknown:			function (chunk) {
			// Unimplemented
			/*switch (chunk.byteLength) {
				// We can tring a string
				default:

				break;
			}
			*/
		}

		, getType:				function (chunk) {
			return Types.getString(chunk.slice(0, 4)).toLowerCase();
		}

	});

	return Types;
});
